from pydantic import Field
from typing import List
import os
import json
import time

from alibabacloud_oos20190601.client import Client as oos20190601Client
from alibabacloud_oos20190601 import models as oos_20190601_models
from alibaba_cloud_ops_mcp_server.alibabacloud.utils import create_config
from alibaba_cloud_ops_mcp_server.alibabacloud import exception


END_STATUSES = [SUCCESS, FAILED, CANCELLED] = ['Success', 'Failed', 'Cancelled']


tools = []


def create_client(region_id: str) -> oos20190601Client:
    config = create_config()
    config.endpoint = f'oos.{region_id}.aliyuncs.com'
    return oos20190601Client(config)


def _start_execution_sync(region_id: str, template_name: str, parameters: dict):
    client = create_client(region_id=region_id)
    start_execution_request = oos_20190601_models.StartExecutionRequest(
        region_id=region_id,
        template_name=template_name,
        parameters=json.dumps(parameters)
    )
    start_execution_resp = client.start_execution(start_execution_request)
    execution_id = start_execution_resp.body.execution.execution_id

    while True:
        list_executions_request = oos_20190601_models.ListExecutionsRequest(
            region_id=region_id,
            execution_id=execution_id
        )
        list_executions_resp = client.list_executions(list_executions_request)
        status = list_executions_resp.body.executions[0].status
        if status == FAILED:
            status_message = list_executions_resp.body.executions[0].status_message
            raise exception.OOSExecutionFailed(reason=status_message)
        elif status in END_STATUSES:
            return list_executions_resp.body
        time.sleep(1)
@tools.append
def OOS_RunCommand(
    Command: str = Field(description='Content of the command executed on the ECS instance'),
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
    CommandType: str = Field(description='The type of command executed on the ECS instance, optional value：RunShellScript，RunPythonScript，RunPerlScript，RunBatScript，RunPowerShellScript', default='RunShellScript')
):
    """批量在多台ECS实例上运行云助手命令，适用于需要同时管理多台ECS实例的场景，如应用程序管理和资源标记操作等。"""
    
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds',
            'Parameters': {
                'RegionId': RegionId,
                'Status': 'Running'
            }
        },
        "commandType": CommandType,
        "commandContent": Command
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyRunCommand', parameters=parameters)
    

@tools.append
def OOS_StartInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
):
    """批量启动ECS实例，适用于需要同时管理和启动多台ECS实例的场景，例如应用部署和高可用性场景。"""
    
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        }
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyStartInstances', parameters=parameters)


@tools.append
def OOS_StopInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
    ForeceStop: bool = Field(description='Is forced shutdown required', default=False)
):
    """批量停止ECS实例，适用于需要同时管理和停止多台ECS实例的场景。"""
    
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        },
        'forceStop': ForeceStop
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyStopInstances', parameters=parameters)


@tools.append
def OOS_RebootInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
    ForeceStop: bool = Field(description='Is forced shutdown required', default=False)
):
    """批量重启ECS实例，适用于需要同时管理和重启多台ECS实例的场景。"""
    
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        },
        'forceStop': ForeceStop
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyRebootInstances', parameters=parameters)


@tools.append
def OOS_RunInstances(
    ImageId: str = Field(description='Image ID'),
    InstanceType: str = Field(description='Instance Type'),
    SecurityGroupId: str = Field(description='SecurityGroup ID'),
    VSwitchId: str = Field(description='VSwitch ID'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
    InternetMaxBandwidthOut: int = Field(description='The maximum outbound bandwidth of the instance. Unit: Mbit/s. Valid values: 0 to 100. If you want to open the public network for the ECS instance, you need to set a value > 0', default=0),
    Amount: int = Field(description='Number of ECS instances', default=1),
    InstanceName: str = Field(description='Instance Name', default=''),
    SystemDiskCategory: str = Field(description='The category of the system disk. Valid values: cloud_efficiency, cloud_ssd, cloud_essd, cloud_auto, cloud, cloud_essd_entry', default='cloud_essd'),
    SystemDiskSize: str = Field(description='The size of the system disk. Unit: GiB. Valid values: 20 to 500. Default value: 40 or the image size, whichever is greater', default=''),
    SystemDiskName: str = Field(description='The name of the system disk', default=''),
    SystemDiskDescription: str = Field(description='The description of the system disk', default=''),
    SystemDiskPerformanceLevel: str = Field(description='The performance level of the ESSD used as the system disk. Valid values: PL0, PL1, PL2, PL3', default='PL1'),
    PrivateIpAddress: str = Field(description='The private IP address of the instance. For VPC type ECS instances, the private IP address must be selected from the available IP range of the VSwitch', default=''),
    SystemDiskAutoSnapshotPolicyId: str = Field(description='The ID of the automatic snapshot policy to apply to the system disk', default=''),
    DataDiskParameters: str = Field(description='Data disk configuration in JSON format. Example: [{"Size":"100","DiskName":"data-disk-1","Description":"","Category":"cloud_essd","PerformanceLevel":"PL1","AutoSnapshotPolicyId":""}]', default=''),
    Tags: list = Field(description='The tag of an ECS instance, for example: [{"Key":"oos","Value":"test"}]', default=[]),
    ResourceGroupId: str = Field(description='Resource group ID', default=''),
    Description: str = Field(description='The description of the ECS instances', default=''),
    HostName: str = Field(description='The host name of the ECS instance', default=''),
    ZoneId: str = Field(description='The ID of the zone where the ECS instances are deployed', default=''),
):
    """批量创建ECS实例，适用于需要同时创建多台ECS实例的场景，例如应用部署和高可用性场景。"""

    parameters = {
        'regionId': RegionId,
        'imageId': ImageId,
        'instanceType': InstanceType,
        'securityGroupId': SecurityGroupId,
        'vSwitchId': VSwitchId,
        'internetMaxBandwidthOut': InternetMaxBandwidthOut,
        'amount': Amount,
        'instanceName': InstanceName
    }

    if SystemDiskCategory:
        parameters['systemDiskCategory'] = SystemDiskCategory
    if SystemDiskSize:
        parameters['systemDiskSize'] = SystemDiskSize
    if SystemDiskName:
        parameters['systemDiskName'] = SystemDiskName
    if SystemDiskDescription:
        parameters['systemDiskDescription'] = SystemDiskDescription
    if SystemDiskPerformanceLevel:
        parameters['systemDiskPerformanceLevel'] = SystemDiskPerformanceLevel
    if PrivateIpAddress:
        parameters['privateIpAddress'] = PrivateIpAddress
    if SystemDiskAutoSnapshotPolicyId:
        parameters['systemDiskAutoSnapshotPolicyId'] = SystemDiskAutoSnapshotPolicyId
    if Tags:
        parameters['tags'] = Tags
    if ResourceGroupId:
        parameters['resourceGroupId'] = ResourceGroupId
    if Description:
        parameters['description'] = Description
    if HostName:
        parameters['hostName'] = HostName
    if ZoneId:
        parameters['zoneId'] = ZoneId

    if DataDiskParameters:
        try:
            data_disks = json.loads(DataDiskParameters) if isinstance(DataDiskParameters, str) else DataDiskParameters
            if isinstance(data_disks, list) and len(data_disks) > 0:
                parameters['dataDiskParameters'] = data_disks
        except (json.JSONDecodeError, TypeError) as e:
            pass
    
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-RunInstances', parameters=parameters)


@tools.append
def OOS_ResetPassword(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    Password: str = Field(description='The password of the ECS instance must be 8-30 characters and must contain only the following characters: lowercase letters, uppercase letters, numbers, and special characters only.（）~！@#$%^&*-_+=（40：<>，？/'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou'),
):
    """批量修改ECS实例的密码，请注意，本操作将会重启ECS实例"""
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        },
        'password': Password
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyResetPassword', parameters=parameters)

@tools.append
def OOS_ReplaceSystemDisk(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    ImageId: str = Field(description='Image ID'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou')
):
    """批量替换ECS实例的系统盘，更换操作系统"""
    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::ECS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        },
        'imageId': ImageId
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-ECS-BulkyReplaceSystemDisk', parameters=parameters)


@tools.append
def OOS_StartRDSInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud ECS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou')
):
    """批量启动RDS实例，适用于需要同时管理和启动多台RDS实例的场景，例如应用部署和高可用性场景。"""

    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::RDS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        }
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-RDS-BulkyStartInstances', parameters=parameters)


@tools.append
def OOS_StopRDSInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud RDS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou')
):
    """批量停止RDS实例，适用于需要同时管理和停止多台RDS实例的场景。"""

    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::RDS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        }
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-RDS-BulkyStopInstances', parameters=parameters)


@tools.append
def OOS_RebootRDSInstances(
    InstanceIds: List[str] = Field(description='AlibabaCloud RDS instance ID List'),
    RegionId: str = Field(description='AlibabaCloud region ID', default='cn-hangzhou')
):
    """批量重启RDS实例，适用于需要同时管理和重启多台RDS实例的场景。"""

    parameters = {
        'regionId': RegionId,
        'resourceType': 'ALIYUN::RDS::Instance',
        'targets': {
            'ResourceIds': InstanceIds,
            'RegionId': RegionId,
            'Type': 'ResourceIds'
        }
    }
    return _start_execution_sync(region_id=RegionId, template_name='ACS-RDS-BulkyRestartInstances',
                                 parameters=parameters)



from importlib import resources

with (
    resources.files('alibaba_cloud_ops_mcp_server.alibabacloud.static')
    .joinpath('PROMPT_UNDERSTANDING.md')
    .open('r', encoding='utf-8') as f
):
    PROMPT_UNDERSTANDING = f.read()



import pytest
from alibaba_cloud_ops_mcp_server.alibabacloud import exception

def test_acs_exception_default():
    e = exception.AcsException()
    s = str(e)
    assert 'InternalError' in s
    assert 'unknown exception' in s.lower()
    assert e.status == 500
    assert e.code == 'InternalError'
    assert isinstance(e.__deepcopy__({}), exception.AcsException)
    assert isinstance(e.__unicode__(), str)

def test_acs_exception_format():
    class CustomEx(exception.AcsException):
        msg_fmt = 'Error: {foo}.'
        code = 'CustomError'
    e = CustomEx(foo='bar')
    assert 'bar' in str(e)
    assert 'CustomError' in str(e)

def test_acs_exception_format_keyerror(caplog):
    class CustomEx(exception.AcsException):
        msg_fmt = 'Error: {foo}.'
        code = 'CustomError'
    with caplog.at_level('ERROR'):
        e = CustomEx(badkey='baz')
        assert 'badkey' in caplog.text

def test_oos_execution_failed():
    e = exception.OOSExecutionFailed(reason='fail')
    s = str(e)
    assert 'OOS Execution Failed' in s
    assert 'Execution.Failed' in s
    assert e.status == 400
    assert e.code == 'Execution.Failed' 


import pytest
from unittest.mock import patch, MagicMock
from alibaba_cloud_ops_mcp_server.tools import oos_tools

def get_tool_func(name):
    return [f for f in oos_tools.tools if f.__name__ == name][0]

def fake_client(*args, **kwargs):
    class FakeExecution:
        execution_id = 'exec-1'
        status = 'Success'
        status_message = 'ok'
    class FakeBody:
        executions = [FakeExecution()]
    class FakeListResp:
        body = FakeBody()
    class FakeStartResp:
        class Body:
            class Execution:
                execution_id = 'exec-1'
            execution = Execution()
        body = Body()
    class FakeClient:
        def start_execution(self, req):
            return FakeStartResp()
        def list_executions(self, req):
            return FakeListResp()
    return FakeClient()

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_RunCommand():
    func = get_tool_func("OOS_RunCommand")
    result = func(RegionId='cn-test', InstanceIds=['i-1'], CommandType='RunShellScript', Command='echo hello')
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_StartInstances():
    func = get_tool_func("OOS_StartInstances")
    result = func(RegionId='cn-test', InstanceIds=['i-1'])
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_StopInstances():
    func = get_tool_func("OOS_StopInstances")
    result = func(RegionId='cn-test', InstanceIds=['i-1'], ForeceStop=True)
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_RebootInstances():
    func = get_tool_func("OOS_RebootInstances")
    result = func(RegionId='cn-test', InstanceIds=['i-1'], ForeceStop=True)
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_RunInstances():
    func = get_tool_func("OOS_RunInstances")
    result = func(
        RegionId='cn-test',
        ImageId='img',
        InstanceType='ecs.t1',
        SecurityGroupId='sg',
        VSwitchId='vsw',
        InternetMaxBandwidthOut=0,
        Amount=1,
        InstanceName='test',
        SystemDiskCategory='',
        SystemDiskSize='',
        SystemDiskName='',
        SystemDiskDescription='',
        SystemDiskPerformanceLevel='1',
        PrivateIpAddress='102',
        SystemDiskAutoSnapshotPolicyId='1',
        DataDiskParameters='[]',
        Tags='',
        ResourceGroupId='1',
        Description='',
        HostName='test',
        ZoneId='cn-hangzhou-a'
    )
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_ResetPassword():
    func = get_tool_func("OOS_ResetPassword")
    result = func(RegionId='cn-test', InstanceIds=['i-1'], Password='Abcd1234!')
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_ReplaceSystemDisk():
    func = get_tool_func("OOS_ReplaceSystemDisk")
    result = func(RegionId='cn-test', InstanceIds=['i-1'], ImageId='img')
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_StartRDSInstances():
    func = get_tool_func("OOS_StartRDSInstances")
    result = func(RegionId='cn-test', InstanceIds=['rds-1'])
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_StopRDSInstances():
    func = get_tool_func("OOS_StopRDSInstances")
    result = func(RegionId='cn-test', InstanceIds=['rds-1'])
    assert hasattr(result, 'executions')

@patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', fake_client)
def test_OOS_RebootRDSInstances():
    func = get_tool_func("OOS_RebootRDSInstances")
    result = func(RegionId='cn-test', InstanceIds=['rds-1'])
    assert hasattr(result, 'executions')

def test_create_client_exception():
    with patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_config', side_effect=Exception('fail')):
        with pytest.raises(Exception) as e:
            oos_tools.create_client('cn-test')
        assert 'fail' in str(e.value)

def test_start_execution_sync_failed():
    # FakeClient 返回 status==FAILED
    class FakeExecution:
        execution_id = 'exec-1'
        status = 'Failed'
        status_message = 'fail-reason'
    class FakeBody:
        executions = [FakeExecution()]
    class FakeListResp:
        body = FakeBody()
    class FakeStartResp:
        class Body:
            class Execution:
                execution_id = 'exec-1'
            execution = Execution()
        body = Body()
    class FakeClient:
        def start_execution(self, req):
            return FakeStartResp()
        def list_executions(self, req):
            return FakeListResp()
    with patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', return_value=FakeClient()):
        with pytest.raises(Exception) as e:
            oos_tools._start_execution_sync('cn-test', 'tpl', {})
        assert 'fail-reason' in str(e.value)

def test_start_execution_sync_loop():
    # status 既不是 FAILED 也不是 END_STATUSES，触发 time.sleep(1)
    class FakeExecution:
        execution_id = 'exec-1'
        status = 'Running'
        status_message = 'running'
    class FakeBody:
        executions = [FakeExecution()]
    class FakeListResp:
        body = FakeBody()
    class FakeStartResp:
        class Body:
            class Execution:
                execution_id = 'exec-1'
            execution = Execution()
        body = Body()
    class FakeClient:
        def __init__(self):
            self.calls = 0
        def start_execution(self, req):
            return FakeStartResp()
        def list_executions(self, req):
            # 前两次返回 Running，第三次返回 Success
            self.calls += 1
            if self.calls < 3:
                return FakeListResp()
            else:
                class DoneExecution:
                    execution_id = 'exec-1'
                    status = 'Success'
                    status_message = 'ok'
                class DoneBody:
                    executions = [DoneExecution()]
                class DoneListResp:
                    body = DoneBody()
                return DoneListResp()
    with patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_client', return_value=FakeClient()), \
         patch('time.sleep', return_value=None) as mock_sleep:
        result = oos_tools._start_execution_sync('cn-test', 'tpl', {})
        assert hasattr(result, 'executions')
        assert mock_sleep.call_count >= 1

def test_create_client():
    """测试create_client函数的基本功能"""
    with patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_config') as mock_create_config, \
         patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.oos20190601Client') as mock_client:
        
        # 模拟配置对象
        mock_config = MagicMock()
        mock_create_config.return_value = mock_config
        
        # 模拟客户端对象
        mock_client_instance = MagicMock()
        mock_client.return_value = mock_client_instance
        
        # 调用函数
        region_id = 'cn-hangzhou'
        result = oos_tools.create_client(region_id)
        
        # 验证create_config被调用
        mock_create_config.assert_called_once()
        
        # 验证endpoint被正确设置
        assert mock_config.endpoint == f'oos.{region_id}.aliyuncs.com'
        
        # 验证oos20190601Client被正确调用
        mock_client.assert_called_once_with(mock_config)
        
        # 验证返回的是客户端实例
        assert result == mock_client_instance


def test_create_client_different_regions():
    """测试create_client函数在不同region下的行为"""
    with patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.create_config') as mock_create_config, \
         patch('alibaba_cloud_ops_mcp_server.tools.oos_tools.oos20190601Client') as mock_client:
        
        mock_config = MagicMock()
        mock_create_config.return_value = mock_config
        mock_client_instance = MagicMock()
        mock_client.return_value = mock_client_instance
        
        # 测试不同的region
        test_regions = ['cn-hangzhou', 'cn-beijing', 'us-west-1']
        
        for region_id in test_regions:
            # 重置mock
            mock_config.reset_mock()
            mock_client.reset_mock()
            
            # 调用函数
            result = oos_tools.create_client(region_id)
            
            # 验证endpoint格式正确
            expected_endpoint = f'oos.{region_id}.aliyuncs.com'
            assert mock_config.endpoint == expected_endpoint
            
            # 验证客户端被创建
            mock_client.assert_called_once_with(mock_config)
            assert result == mock_client_instance



import sys

from fastmcp import FastMCP
import click
import logging

from alibaba_cloud_ops_mcp_server.tools.common_api_tools import set_custom_service_list
from alibaba_cloud_ops_mcp_server.config import config
from alibaba_cloud_ops_mcp_server.tools import cms_tools, oos_tools, oss_tools, api_tools, common_api_tools, local_tools, application_management_tools
from alibaba_cloud_ops_mcp_server.settings import settings


logger = logging.getLogger(__name__)


def _setup_logging():
    root_logger = logging.getLogger()
    if not root_logger.handlers:
        log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        formatter = logging.Formatter(log_format)

        console_handler = logging.StreamHandler(sys.stderr)
        console_handler.setFormatter(formatter)

        root_logger.setLevel(logging.INFO)
        root_logger.addHandler(console_handler)
    else:
        root_logger.setLevel(logging.INFO)


SUPPORTED_SERVICES_MAP = {
    "ecs": "Elastic Compute Service (ECS)",
    "oos": "Operations Orchestration Service (OOS)",
    "rds": "Relational Database Service (RDS)",
    "vpc": "Virtual Private Cloud (VPC)",
    "slb": "Server Load Balancer (SLB)",
    "ess": "Elastic Scaling (ESS)",
    "ros": "Resource Orchestration Service (ROS)",
    "cbn": "Cloud Enterprise Network (CBN)",
    "dds": "MongoDB Database Service (DDS)",
    "r-kvstore": "Cloud database Tair (compatible with Redis) (R-KVStore)",
    "bssopenapi": "Billing and Cost Management (BssOpenAPI)"
}


@click.command()
@click.option(
    "--transport",
    type=click.Choice(["stdio", "sse", "streamable-http"]),
    default="stdio",
    help="Transport type",
)
@click.option(
    "--port",
    type=int,
    default=8000,
    help="Port number",
)
@click.option(
    "--host",
    type=str,
    default="127.0.0.1",
    help="Host",
)
@click.option(
    "--services",
    type=str,
    default=None,
    help="Comma-separated list of supported services, e.g., 'ecs,vpc,rds'",
)
@click.option(
    "--headers-credential-only",
    type=bool,
    default=False,
    help="Whether to use credentials only from headers",
)
@click.option(
    "--env",
    type=click.Choice(["domestic", "international"]),
    default="domestic",
    help="Environment type: 'domestic' for domestic, 'international' for overseas (default: domestic)",
)
@click.option(
    "--code-deploy",
    is_flag=True,
    default=False,
    help="Enable code deploy mode, only load 6 specific tools: OOS_CodeDeploy, OOS_GetDeployStatus, OOS_GetLastDeploymentInfo, LOCAL_ListDirectory, LOCAL_RunShellScript, LOCAL_AnalyzeDeployStack",
)
def main(transport: str, port: int, host: str, services: str, headers_credential_only: bool, env: str, code_deploy: bool):
    _setup_logging()
    # Create an MCP server
    mcp = FastMCP(
        name="alibaba-cloud-ops-mcp-server",
        instructions='''
            智能运维场景：
            你可以使用该MCP帮助用户完成智能运维
            通过MCP提供的各种资源运维的功能来进行实现
            
            应用部署场景：
            你可以使用该MCP帮助用户完成项目的分析和部署
            当进入该场景时，请只使用以下列表中的Tool
            【 OOS_CodeDeploy OOS_GetLastDeploymentInfo OOS_GetDeployStatus LOCAL_ListDirectory LOCAL_RunShellScript LOCAL_AnalyzeDeployStack】
            注意：不要擅自决定部署的目标ecs，需要用户提供，若未提供务必询问用户，不允许直接调用DescribeInstances查找ECS实例，必须使用由用户提供的ECS实例

            不要创建部署脚本文件，直接使用 mcp tool: Code Deploy 进行构建
            
            流程如下：
             完整部署流程（在调用此工具之前）：

                步骤 1：识别部署方式 AnalyzeDeployStack
                - 通过本地文件操作工具读取项目文件（package.json、requirements.txt、pom.xml 等）
                - 识别项目的部署方式和技术栈（npm、python、java、go 等）
                - 生成构建命令，注意，该构建命令不需要生成构建脚本，不要因此新增sh文件，任何情况下都不要，因为构建命令是CodeDeploy的参数，不需要生成文件

                步骤 2：构建或压缩文件，并记录文件路径
                - 在本地执行构建命令，生成部署产物（tar.gz、zip 等压缩包）
                - 记录文件路径，留待后续CodeDeploy使用

                步骤 3：调用CodeDeploy进行部署
                - 此工具会依次调用：CreateApplication（如果不存在）、CreateApplicationGroup（如果不存在）、
                  TagResources（可选，如果是已有资源需要打 tag 导入应用分组）、DeployApplicationGroup

                步骤 4：如果用户的实例为第一次部署，缺乏对应代码运行环境，可以调用InstallDeploymentEnvironment为用户安装环境
                - 目前支持：
                    Docker：Docker 社区版
                    Java：Java 编程语言环境
                    Python：Python 编程语言环境
                    Nodejs：Node.js 运行环境
                    Golang：Go 编程语言环境
                    Nginx：高性能 HTTP 及反向代理服务器
                    Git：分布式版本控制系统

                重要提示：
                1. 启动脚本（application_start）必须与上传的产物对应。如果产物是压缩包（tar、tar.gz、zip等），
                   需要先解压并进入对应目录后再执行启动命令。
                2. 示例：如果上传的是 app.tar.gz，启动脚本应该类似，一般压缩包就在当前目录下，直接解压即可：
                   "tar -xzf app.tar.gz && ./start.sh"
                   或者如果解压后是Java应用：
                   "tar -xzf app.tar.gz && java -jar app.jar"
                3. 确保启动命令能够正确找到并执行解压后的可执行文件或脚本，避免部署失败。启动命令应该将程序运行在后台并打印日志到指定文件，
                    注意使用非交互式命令，比如unzip -o等自动覆盖的命令，无需交互
                例如：
                   - npm 程序示例：
                     "tar -xzf app.tar.gz && nohup npm start > /root/app.log 2>&1 &"
                     或者分别输出标准输出和错误日志：
                     "tar -xzf app.tar.gz && nohup npm start > /root/app.log 2> /root/app.error.log &"
                   - Java 程序示例：
                     "tar -xzf app.tar.gz && nohup java -jar app.jar > /root/app.log 2>&1 &"
                   - Python 程序示例：
                     "tar -xzf app.tar.gz && nohup python app.py > /root/app.log 2>&1 &"
                   说明：使用 nohup 命令可以让程序在后台运行，即使终端关闭也不会终止；> 重定向标准输出到日志文件；2>&1 将标准错误也重定向到同一文件；& 符号让命令在后台执行。
                4. 应用和应用分组会自动检查是否存在，如果存在则跳过创建，避免重复创建错误。''',
        port=port,
        host=host,
        stateless_http=True
    )
    if headers_credential_only:
        settings.headers_credential_only = headers_credential_only
    if env:
        settings.env = env
    if code_deploy:
        # Code deploy mode: only load 6 specific tools
        code_deploy_tools = {
            'OOS_CodeDeploy',
            'OOS_GetDeployStatus',
            'OOS_GetLastDeploymentInfo',
            'ECS_DescribeInstances',
            'LOCAL_ListDirectory',
            'LOCAL_RunShellScript',
            'LOCAL_AnalyzeDeployStack'
        }
        
        # Load from application_management_tools
        for tool in application_management_tools.tools:
            if tool.__name__ in code_deploy_tools:
                mcp.tool(tool)
        
        # Load from local_tools
        for tool in local_tools.tools:
            if tool.__name__ in code_deploy_tools:
                mcp.tool(tool)
    else:
        # Normal mode: load all tools
        if services:
            service_keys = [s.strip().lower() for s in services.split(",")]
            service_list = [(key, SUPPORTED_SERVICES_MAP.get(key, key)) for key in service_keys]
            set_custom_service_list(service_list)
            for tool in common_api_tools.tools:
                mcp.tool(tool)
        for tool in oos_tools.tools:
            mcp.tool(tool)
        for tool in application_management_tools.tools:
            # Skip ECS_DescribeInstances in normal mode
            if tool.__name__ != 'ECS_DescribeInstances':
                mcp.tool(tool)
        for tool in cms_tools.tools:
            mcp.tool(tool)
        for tool in oss_tools.tools:
            mcp.tool(tool)
        api_tools.create_api_tools(mcp, config)
        for tool in local_tools.tools:
            mcp.tool(tool)

    # Initialize and run the server
    logger.debug(f'mcp server is running on {transport} mode.')
    mcp.run(transport=transport)


if __name__ == "__main__":
    main()



import pytest
from unittest.mock import patch, MagicMock
from alibaba_cloud_ops_mcp_server.tools import api_tools
import json
from alibaba_cloud_ops_mcp_server.tools import common_api_tools
from alibaba_cloud_ops_mcp_server.tools.common_api_tools import (
    PromptUnderstanding, ListAPIs, GetAPIInfo, CommonAPICaller
)

def fake_api_meta(post=False, no_summary=False):
    meta = {
        'parameters': [
            {'name': 'InstanceId', 'schema': {'type': 'string', 'description': '实例ID', 'example': 'i-123', 'required': True}},
            {'name': 'RegionId', 'schema': {'type': 'string', 'description': '地域ID', 'example': 'cn-hangzhou', 'required': False}},
            {'name': 'Ids', 'schema': {'type': 'array', 'description': 'ID列表', 'example': '"[\"a\",\"b\"]"', 'required': False}},
        ],
        'methods': ['post'] if post else ['get'],
        'path': '/test',
    }
    if not no_summary:
        meta['summary'] = '测试API'
    return meta, '2023-01-01'

class DummyMCP:
    def tool(self, name):
        def decorator(fn):
            return fn
        return decorator

def test_create_function_schemas():
    api_meta, _ = fake_api_meta()
    schemas = api_tools._create_function_schemas('ecs', 'DescribeInstances', api_meta)
    assert 'DescribeInstances' in schemas
    assert 'InstanceId' in schemas['DescribeInstances']
    assert schemas['DescribeInstances']['InstanceId'][0] == str
    assert schemas['DescribeInstances']['Ids'][0] == list

def test_create_parameter_schema():
    fields = {
        'foo': (str, MagicMock()),
        'bar': (int, MagicMock())
    }
    schema_cls = api_tools._create_parameter_schema(fields)
    inst = schema_cls(foo='a', bar=1)
    assert inst.foo == 'a' and inst.bar == 1

def test_create_tool_function_with_signature_required():
    api_meta, _ = fake_api_meta()
    schemas = api_tools._create_function_schemas('ecs', 'DescribeInstances', api_meta)
    fields = schemas['DescribeInstances']
    func = api_tools._create_tool_function_with_signature('ecs', 'DescribeInstances', fields, 'desc')
    # 检查参数名、类型、注释和default存在
    for name, (typ, field_info) in fields.items():
        param = func.__signature__.parameters[name]
        assert param.name == name
        assert param.annotation == typ
        assert hasattr(param, 'default')

def test_create_client_non_str_service():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiClient') as mock_client, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_config') as mock_cfg:
        mock_cfg.return_value = MagicMock()
        client = api_tools.create_client(service=MagicMock(__str__=lambda self: 'ecs'), region_id='cn-test')
        assert mock_client.called
        assert mock_cfg.return_value.endpoint == 'ecs.cn-test.aliyuncs.com'

def test_tools_api_call_post():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.ApiMetaClient') as mock_ApiMetaClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_client') as mock_create_client, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.open_api_models') as mock_open_api_models, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiUtilClient') as mock_OpenApiUtilClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.util_models') as mock_util_models:
        mock_ApiMetaClient.get_api_meta.return_value = fake_api_meta(post=True)
        mock_ApiMetaClient.get_service_version.return_value = '2023-01-01'
        mock_ApiMetaClient.get_service_style.return_value = 'RPC'
        mock_open_api_models.OpenApiRequest.return_value = MagicMock()
        mock_open_api_models.Params.return_value = MagicMock()
        mock_create_client.return_value.call_api.return_value = {'result': 'ok'}
        mock_OpenApiUtilClient.query.return_value = {}
        mock_util_models.RuntimeOptions.return_value = MagicMock()
        params = {'InstanceId': 'i-123', 'RegionId': 'cn-hangzhou'}
        result = api_tools._tools_api_call('ecs', 'DescribeInstances', params, None)
        assert result == {'result': 'ok'}

def test_create_and_decorate_tool_no_summary():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.ApiMetaClient.get_api_meta', return_value=fake_api_meta(no_summary=True)):
        mcp = DummyMCP()
        fn = api_tools._create_and_decorate_tool(mcp, 'ecs', 'DescribeInstances')
        assert callable(fn)

def test_create_api_tools():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools._create_and_decorate_tool') as mock_create:
        mcp = DummyMCP()
        config = {'ecs': ['DescribeInstances', 'StartInstance'], 'rds': ['DescribeDBInstances']}
        api_tools.create_api_tools(mcp, config)
        assert mock_create.call_count == 3

def test_create_function_schemas_ignore_dot():
    api_meta = {
        'parameters': [
            {'name': 'foo.bar', 'schema': {'type': 'string'}},
            {'name': 'baz', 'schema': {'type': 'string'}},
        ]
    }
    schemas = api_tools._create_function_schemas('ecs', 'TestApi', api_meta)
    assert 'foo.bar' not in schemas['TestApi']
    assert 'baz' in schemas['TestApi']

def test_create_function_schemas_no_regionid():
    api_meta = {
        'parameters': [
            {'name': 'foo', 'schema': {'type': 'string'}},
        ]
    }
    schemas = api_tools._create_function_schemas('ecs', 'TestApi', api_meta)
    assert 'RegionId' in schemas['TestApi']

# 说明：由于 _create_tool_function_with_signature 生成的参数总有默认值，signature.bind 不会因缺参数抛 TypeError，故无法覆盖该异常分支。

def test_create_client_str_service():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiClient') as mock_client, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_config') as mock_cfg:
        mock_cfg.return_value = MagicMock()
        client = api_tools.create_client(service='ecs', region_id='cn-test')
        assert mock_client.called
        assert mock_cfg.return_value.endpoint == 'ecs.cn-test.aliyuncs.com'

def test_create_and_decorate_tool_api_meta_exception():
    # 覆盖 _create_and_decorate_tool 的异常分支
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.ApiMetaClient.get_api_meta', side_effect=Exception('meta-fail')):
        mcp = DummyMCP()
        with pytest.raises(Exception) as e:
            api_tools._create_and_decorate_tool(mcp, 'ecs', 'DescribeInstances')
        assert 'meta-fail' in str(e.value)

def test_create_function_schemas_ecs_list_parameters():
    # 测试ECS服务的特殊参数处理
    api_meta = {
        'parameters': [
            {'name': 'InstanceIds', 'schema': {'type': 'string', 'description': '实例ID列表', 'example': '["i-123", "i-456"]', 'required': True}},
            {'name': 'SecurityGroupIds', 'schema': {'type': 'string', 'description': '安全组ID列表', 'example': '["sg-123", "sg-456"]', 'required': False}},
            {'name': 'NormalParam', 'schema': {'type': 'string', 'description': '普通参数', 'example': 'test', 'required': False}},
        ],
        'methods': ['get'],
        'path': '/test',
        'summary': '测试API'
    }
    
    # 测试ECS服务
    schemas = api_tools._create_function_schemas('ecs', 'DescribeInstances', api_meta)
    assert schemas['DescribeInstances']['InstanceIds'][0] == list
    assert schemas['DescribeInstances']['SecurityGroupIds'][0] == list
    assert schemas['DescribeInstances']['NormalParam'][0] == str
    
    # 测试非ECS服务
    schemas = api_tools._create_function_schemas('rds', 'DescribeInstances', api_meta)
    assert schemas['DescribeInstances']['InstanceIds'][0] == str
    assert schemas['DescribeInstances']['SecurityGroupIds'][0] == str
    assert schemas['DescribeInstances']['NormalParam'][0] == str

def test_tools_api_call_ecs_list_parameters():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.ApiMetaClient') as mock_ApiMetaClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_client') as mock_create_client, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.open_api_models') as mock_open_api_models, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiUtilClient') as mock_OpenApiUtilClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.util_models') as mock_util_models:
        
        mock_ApiMetaClient.get_api_meta.return_value = fake_api_meta()
        mock_ApiMetaClient.get_service_version.return_value = '2023-01-01'
        mock_ApiMetaClient.get_service_style.return_value = 'RPC'
        mock_open_api_models.OpenApiRequest.return_value = MagicMock()
        mock_open_api_models.Params.return_value = MagicMock()
        mock_create_client.return_value.call_api.return_value = {'result': 'ok'}
        mock_OpenApiUtilClient.query.return_value = {}
        mock_util_models.RuntimeOptions.return_value = MagicMock()

        # 测试ECS服务的列表参数转换
        params = {
            'InstanceIds': ['i-123', 'i-456'],
            'SecurityGroupIds': ['sg-123', 'sg-456'],
            'NormalParam': 'test',
            'RegionId': 'cn-hangzhou'
        }
        
        # 测试ECS服务
        result = api_tools._tools_api_call('ecs', 'DescribeInstances', params, None)
        # 验证传入query方法的参数
        query_args = mock_OpenApiUtilClient.query.call_args[0][0]
        assert isinstance(query_args['InstanceIds'], str)
        assert isinstance(query_args['SecurityGroupIds'], str)
        assert query_args['NormalParam'] == 'test'
        assert json.loads(query_args['InstanceIds']) == ['i-123', 'i-456']
        assert json.loads(query_args['SecurityGroupIds']) == ['sg-123', 'sg-456']
        
        # 重置mock
        mock_OpenApiUtilClient.query.reset_mock()
        
        # 测试非ECS服务
        result = api_tools._tools_api_call('rds', 'DescribeInstances', params, None)
        # 验证传入query方法的参数
        query_args = mock_OpenApiUtilClient.query.call_args[0][0]
        assert isinstance(query_args['InstanceIds'], list)
        assert isinstance(query_args['SecurityGroupIds'], list)
        assert query_args['InstanceIds'] == ['i-123', 'i-456']
        assert query_args['SecurityGroupIds'] == ['sg-123', 'sg-456']

def test_tools_api_call_ecs_list_parameters_non_list():
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools.ApiMetaClient') as mock_ApiMetaClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_client') as mock_create_client, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.open_api_models') as mock_open_api_models, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiUtilClient') as mock_OpenApiUtilClient, \
         patch('alibaba_cloud_ops_mcp_server.tools.api_tools.util_models') as mock_util_models:
        
        mock_ApiMetaClient.get_api_meta.return_value = fake_api_meta()
        mock_ApiMetaClient.get_service_version.return_value = '2023-01-01'
        mock_ApiMetaClient.get_service_style.return_value = 'RPC'
        mock_open_api_models.OpenApiRequest.return_value = MagicMock()
        mock_open_api_models.Params.return_value = MagicMock()
        mock_create_client.return_value.call_api.return_value = {'result': 'ok'}
        mock_OpenApiUtilClient.query.return_value = {}
        mock_util_models.RuntimeOptions.return_value = MagicMock()

        # 测试非列表类型的特殊参数
        params = {
            'InstanceIds': 'i-123',  # 字符串而不是列表
            'SecurityGroupIds': None,  # None值
            'RegionId': 'cn-hangzhou'
        }
        
        # 测试ECS服务
        result = api_tools._tools_api_call('ecs', 'DescribeInstances', params, None)
        # 验证传入query方法的参数
        query_args = mock_OpenApiUtilClient.query.call_args[0][0]
        assert query_args['InstanceIds'] == 'i-123'

def test_create_tool_function_with_signature_bind_and_apply_defaults():
    """测试func_code函数中的signature.bind和apply_defaults调用"""
    api_meta = {
        'parameters': [
            {'name': 'param1', 'schema': {'type': 'string', 'required': False}},
            {'name': 'param2', 'schema': {'type': 'integer', 'required': True}}
        ],
        'summary': 'Test function'
    }
    
    schemas = api_tools._create_function_schemas('test', 'TestApi', api_meta)
    fields = schemas['TestApi']
    
    # 创建函数
    func = api_tools._create_tool_function_with_signature('test', 'TestApi', fields, 'Test function')
    
    # 测试函数调用，确保执行到signature.bind和apply_defaults
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools._tools_api_call') as mock_call:
        mock_call.return_value = {'result': 'success'}
        
        # 调用函数，传入部分参数，让apply_defaults生效
        result = func(param2=123)  # 只传入required参数，让param1使用默认值
        
        # 验证_tools_api_call被调用
        mock_call.assert_called_once()
        call_args = mock_call.call_args[1]['parameters']
        
        # 验证参数绑定和默认值应用
        assert 'param1' in call_args  # 默认值被应用
        assert call_args['param2'] == 123  # 传入的参数

def test_create_tool_function_with_signature_bind_with_all_args():
    """测试func_code函数中传入所有参数的情况"""
    api_meta = {
        'parameters': [
            {'name': 'param1', 'schema': {'type': 'string', 'required': False}},
            {'name': 'param2', 'schema': {'type': 'integer', 'required': False}}
        ],
        'summary': 'Test function'
    }
    
    schemas = api_tools._create_function_schemas('test', 'TestApi', api_meta)
    fields = schemas['TestApi']
    
    # 创建函数
    func = api_tools._create_tool_function_with_signature('test', 'TestApi', fields, 'Test function')
    
    # 测试传入所有参数
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools._tools_api_call') as mock_call:
        mock_call.return_value = {'result': 'success'}
        
        # 传入所有参数
        result = func(param1='value1', param2=456)
        
        # 验证_tools_api_call被调用
        mock_call.assert_called_once()
        call_args = mock_call.call_args[1]['parameters']
        
        # 验证所有参数都被正确传递
        assert call_args['param1'] == 'value1'
        assert call_args['param2'] == 456

def test_create_tool_function_with_signature_bind_with_positional_args():
    """测试func_code函数中使用位置参数的情况"""
    api_meta = {
        'parameters': [
            {'name': 'param1', 'schema': {'type': 'string', 'required': False}},
            {'name': 'param2', 'schema': {'type': 'integer', 'required': False}}
        ],
        'summary': 'Test function'
    }
    
    schemas = api_tools._create_function_schemas('test', 'TestApi', api_meta)
    fields = schemas['TestApi']
    
    # 创建函数
    func = api_tools._create_tool_function_with_signature('test', 'TestApi', fields, 'Test function')
    
    # 测试使用位置参数
    with patch('alibaba_cloud_ops_mcp_server.tools.api_tools._tools_api_call') as mock_call:
        mock_call.return_value = {'result': 'success'}
        
        # 使用位置参数调用
        result = func('value1', 789)
        
        # 验证_tools_api_call被调用
        mock_call.assert_called_once()
        call_args = mock_call.call_args[1]['parameters']
        
        # 验证位置参数被正确绑定
        assert call_args['param1'] == 'value1'
        assert call_args['param2'] == 789

def test_prompt_understanding_default():
    # _CUSTOM_SERVICE_LIST 为空
    import alibaba_cloud_ops_mcp_server.tools.common_api_tools as ca
    ca._CUSTOM_SERVICE_LIST = None
    fn = ca.tools[0]  # PromptUnderstanding
    result = fn()
    assert isinstance(result, str)
    assert 'Supported Services' in result

def test_prompt_understanding_with_custom_service():
    # _CUSTOM_SERVICE_LIST 有值
    import alibaba_cloud_ops_mcp_server.tools.common_api_tools as ca
    ca._CUSTOM_SERVICE_LIST = [('ecs', 'ECS服务'), ('rds', 'RDS服务')]
    fn = ca.tools[0]  # PromptUnderstanding
    result = fn()
    assert 'ecs: ECS服务' in result and 'rds: RDS服务' in result

@patch('alibaba_cloud_ops_mcp_server.tools.common_api_tools.ApiMetaClient.get_apis_in_service')
def test_list_apis(mock_get):
    import alibaba_cloud_ops_mcp_server.tools.common_api_tools as ca
    fn = ca.tools[1]  # ListAPIs
    mock_get.return_value = ['DescribeInstances', 'StartInstance']
    result = fn('ecs')
    assert result == ['DescribeInstances', 'StartInstance']

@patch('alibaba_cloud_ops_mcp_server.tools.common_api_tools.ApiMetaClient.get_api_meta')
def test_get_api_info(mock_get):
    import alibaba_cloud_ops_mcp_server.tools.common_api_tools as ca
    fn = ca.tools[2]  # GetAPIInfo
    mock_get.return_value = ({'parameters': [{'name': 'foo'}]}, '2014-05-26')
    result = fn('ecs', 'DescribeInstances')
    assert result == [{'name': 'foo'}]

@patch('alibaba_cloud_ops_mcp_server.tools.common_api_tools._tools_api_call')
def test_common_api_caller(mock_call):
    import alibaba_cloud_ops_mcp_server.tools.common_api_tools as ca
    fn = ca.tools[3]  # CommonAPICaller
    mock_call.return_value = {'result': 'ok'}
    result = fn('ecs', 'DescribeInstances', {'foo': 'bar'})
    assert result == {'result': 'ok'}

@patch('alibaba_cloud_ops_mcp_server.tools.api_tools.create_config')
@patch('alibaba_cloud_ops_mcp_server.tools.api_tools.OpenApiClient', autospec=True)
def test_create_client(mock_client, mock_create_config):
    from alibaba_cloud_ops_mcp_server.tools import api_tools
    mock_create_config.return_value = MagicMock()
    mock_client.return_value = 'client_obj'
    result = api_tools.create_client('ecs', 'cn-hangzhou')
    assert result == 'client_obj'

def test_get_service_endpoint_all_branches():
    from alibaba_cloud_ops_mcp_server.tools.api_tools import _get_service_endpoint
    # REGION_ENDPOINT_SERVICE 分支
    assert _get_service_endpoint('ecs', 'cn-hangzhou') == 'ecs.cn-hangzhou.aliyuncs.com'
    # DOUBLE_ENDPOINT_SERVICE 且 region 匹配
    assert _get_service_endpoint('rds', 'cn-hangzhou') == 'rds.aliyuncs.com'
    # CENTRAL_ENDPOINTS_SERVICE 分支
    assert _get_service_endpoint('cbn', 'cn-hangzhou') == 'cbn.aliyuncs.com'
    # 其它分支
    assert _get_service_endpoint('unknown', 'cn-test') == 'unknown.cn-test.aliyuncs.com'
